@@ -3,6 +3,10 @@
  *
  * Copyright (c) 2012 Cypress Semiconductor Corporation.
  *
+ * Additional contributors include:
+ *   Kamal Mostafa <kamal@canonical.com>
+ *   Kyle Fazzari <git@status.e4ward.com>
+ *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
  * the Free Software Foundation.
@@ -264,6 +268,13 @@ static int cypress_read_fw_version(struct psmouse *psmouse)
 	cytp->fw_version = param[2] & FW_VERSION_MASX;
 	cytp->vital_statics_supported = (param[2] & VITAL_STATICS_MASK) ? 1 : 0;
 
+	/*
+	 * Trackpad fw_version 11 (in Dell XPS12) yields a bogus response to
+	 * CYTP_CMD_READ_VITAL_STATISTICS so do not try to use it. LP: #1103594.
+	 */
+	if (cytp->fw_version >= 11)
+		cytp->vital_statics_supported = 0;
+
 	cytp_dbg("cytp->fw_version = %d\n", cytp->fw_version);
 	cytp_dbg("cytp->vital_statics_supported = %d\n",
 		 cytp->vital_statics_supported);
@@ -285,6 +296,9 @@ static int cypress_read_vital_statistics(struct psmouse *psmouse)
 	cytp->tp_res_x = cytp->tp_max_abs_x / cytp->tp_width;
 	cytp->tp_res_y = cytp->tp_max_abs_y / cytp->tp_high;
 
+	if (!cytp->vital_statics_supported)
+		return 0;
+
 	memset(param, 0, sizeof(param));
 	if (cypress_send_ext_cmd(psmouse, CYTP_CMD_READ_VITAL_STATISTICS, param) == 0) {
 		/* Update trackpad parameters. */
@@ -346,15 +360,14 @@ static int cypress_read_vital_statistics(struct psmouse *psmouse)
 
 static int cypress_query_hardware(struct psmouse *psmouse)
 {
-	struct cytp_data *cytp = psmouse->private;
+	int ret;
 
 	if (cypress_read_fw_version(psmouse))
 		return -1;
 
-	if (cytp->vital_statics_supported) {
-		if (cypress_read_vital_statistics(psmouse))
-			return -1;
-	}
+	ret = cypress_read_vital_statistics(psmouse);
+	if (ret)
+		return -1;
 
 	return 0;
 }
@@ -478,8 +491,8 @@ static int cypress_get_finger_count(unsigned char header_byte)
 				if (finger_count == 2) {
 					finger_count = 5;
 				} else {
-					/* invalid header byte data, set invliad finger count. */
-					finger_count = 6;
+					/* Invalid contact (e.g. palm). Ignore it. */
+					finger_count = 0;
 				}
 			}
 		}
@@ -622,13 +635,12 @@ static int cypress_parse_packet(const unsigned char packet[],
 			cytp_dbg("cypress_parse_packet: received invalid packet.\n");
 		}
 
+		report_data->tap = (header_byte & ABS_MULTIFINGER_TAP) ? 1 : 0;
+
 		/* Remove HSCROLL bit */
 		if (report_data->contact_cnt == 4)
 			header_byte &= ~(ABS_HSCROLL_BIT);
 
-		if (report_data->contact_cnt > 0)
-			report_data->tap = (header_byte & ABS_MULTIFINGER_TAP) ? 1 : 0;
-
 		if (report_data->contact_cnt == 1) {
 			report_data->contacts[0].x =
 				((packet[1] & 0x70) << 4) | packet[2];
