@@ -102,7 +102,6 @@ struct fw_rsc_hdr {
  *		    the remote processor will be writing logs.
  * @RSC_VDEV:       declare support for a virtio device, and serve as its
  *		    virtio header.
- * @RSC_SUSPD_TIME: suspend timeout used for autosuspend feature
  * @RSC_LAST:       just keep this one at the end
  *
  * For more details regarding a specific resource type, please see its
@@ -118,8 +117,7 @@ enum fw_resource_type {
 	RSC_DEVMEM	= 1,
 	RSC_TRACE	= 2,
 	RSC_VDEV	= 3,
-	RSC_SUSPD_TIME	= 4,
-	RSC_LAST	= 5,
+	RSC_LAST	= 4,
 };
 
 #define FW_RSC_ADDR_ANY (0xFFFFFFFFFFFFFFFF)
@@ -309,14 +307,6 @@ struct fw_rsc_vdev {
 	struct fw_rsc_vdev_vring vring[0];
 } __packed;
 
-/**
- * struct fw_rsc_suspd_time - suspend timeout resource
- * @suspd_time: auto suspend timeout
- */
-struct fw_rsc_suspd_time {
-	s32 suspd_time;
-} __packed;
-
 /**
  * struct rproc_mem_entry - memory entry descriptor
  * @va:	virtual address
@@ -342,8 +332,6 @@ struct rproc;
  * @start:	power on the device and boot it
  * @stop:	power off the device
  * @kick:	kick a virtqueue (virtqueue id given as a parameter)
- * @suspend:	suspend callback (auto_suspend flag as a parameter)
- * @resume:	resume callback
  * @set_latency		set latency on remote processor
  * @set_bandwidth	set bandwidth on remote processor
  * @set_frequency	set frequency of remote processor
@@ -352,8 +340,6 @@ struct rproc_ops {
 	int (*start)(struct rproc *rproc);
 	int (*stop)(struct rproc *rproc);
 	void (*kick)(struct rproc *rproc, int vqid);
-	int (*suspend)(struct rproc *rproc, bool auto_suspend);
-	int (*resume)(struct rproc *rproc);
 	int (*set_latency)(struct device *dev, struct rproc *rproc, long v);
 	int (*set_bandwidth)(struct device *dev, struct rproc *rproc, long v);
 	int (*set_frequency)(struct device *dev, struct rproc *rproc, long v);
@@ -430,10 +416,6 @@ enum rproc_err {
  * @error_handler: workqueue for reseting virtio devices
  * @crash_cnt: counter for fatal errors
  * @recovery_disabled: flag that state if recovery was disabled
- * @index: index of this rproc device
- * @auto_suspend_timeout: store the auto suspend timeout for a rproc in msecs
- * @need resume: if true a resume is needed in the system resume callback
- * @system_suspended: true if a system suspend has happened
  */
 struct rproc {
 	struct klist_node node;
@@ -442,7 +424,7 @@ struct rproc {
 	const char *firmware;
 	void *priv;
 	const struct rproc_ops *ops;
-	struct device dev;
+	struct device *dev;
 	struct kref refcount;
 	atomic_t power;
 	unsigned int state;
@@ -459,10 +441,6 @@ struct rproc {
 	struct work_struct error_handler;
 	unsigned crash_cnt;
 	bool recovery_disabled;
-	int index;
-	int auto_suspend_timeout;
-	bool need_resume;
-	bool system_suspended;
 };
 
 /* we currently support only two vrings per rvdev */
